/**
 * @file
 * Script for the SiteAttention plugin.
 */

var SiteAttentionDrupal = function ( data )
{
    /**
     * The implementation of the FieldAbstract class specific for Drupal 7 image
     * field. This acts as a wrapper and listener for the `Image Add Widget`.
     * This class inherits FieldAbstract and uses the same parameters for it's
     * constructor.
     *
     * @class      FieldDrupal7Image Drupal 7 image field type
     */

    var FieldDrupal7Image = function () {

        function FieldDrupal7Image(field){

            const local = SiteAttentionModule.FieldAbstract.call(this,field);

            local.load = function ()
            {
                if (!local.this._find()) return false;

                if (!local.element.getElementsByClassName('image-widget-data'))
                {
                    console.warn('SiteAttention: Field <' +local.name+ '> <' +local.selector[0]+ '> could not be resolved to FieldDrupal7Image.');
                }

                local.object = new parent.MutationObserver(local.listener);
                local.container = local.element;

                local.listener();
                local.bind();

                return true;
            }

            local.bind = function ()
            {
                local.alt = SA.lib.salt.one(local.selector[0] + '-und-0-alt');
                local.removeButton = SA.lib.salt.one(local.selector[0] + '-und-0-remove-button');

                if (local.alt)
                {
                    local.alt.addEventListener('keyup',local.listener);
                }
                else if (local.removeButton && !local.element.getElementsByClassName('siteattention-imagewidget-alt-warn')[0])
                {
                    var widgetData = local.element.getElementsByClassName('image-widget-data')[0],
                        altReplace = document.createElement('div'),
                        field_machine = local.element.id.split('-').join('_').slice(5),
                        manageFields = '/admin/structure/types/manage/' +SA.doc.type+ '/fields/' +field_machine,
                        link = document.createElement('a');

                    altReplace.className = 'siteattention-imagewidget-alt-warn';
                    altReplace.innerHTML = 'The <b>Preview</b> and <b>Alternate text</b> must be '; //<a target="_blank" href="${manageFields}">enabled</a>

                    link.text = 'enabled';
                    link.href = '#';
                    link.addEventListener('click', function(event){
                        event.preventDefault();
                        parent.open(manageFields,'_blank');
                        return false;
                    }, true);

                    altReplace.appendChild(link);
                    altReplace.appendChild(document.createTextNode('!'));

                    widgetData.appendChild(altReplace);
                }

                local.object.observe(local.element,{
                    childList: true,
                    attributes: false,
                    characterData: false,
                    subtree: true,
                    attributeOldValue: false,
                    characterDataOldValue: false,
                });
            }

            local.listener = function (mutations,observer)
            {
                if (observer instanceof parent.MutationObserver) // overlay issues so use parent
                {
                    const mutation = mutations[0];

                    if
                    (
                        mutation.type === 'childList' &&
                        mutation.removedNodes.length &&
                        mutation.removedNodes[0].className === 'ajax-progress ajax-progress-throbber'
                    )
                    {
                        local.bind();
                        local.this._update_timer.call(local.this);
                    }
                }

                else
                {
                    local.this._update_timer.call(local.this);
                }
            }

            local.update = function ()
            {
                const imgs = local.element.getElementsByTagName('img');

                for (var i = imgs.length - 1; i >= 0; i--) {
                    var img = imgs[i];

                    if
                    (   ! img.attributes.hasOwnProperty('typeof') ||
                        ! img.attributes.typeof.value === 'foaf:Image'
                    ) return;

                    const imgClone = img.cloneNode();

                    if
                    (
                        local.alt &&
                        local.alt.value && !
                        (
                            imgClone.attributes.hasOwnProperty('alt') &&
                            imgClone.attributes.alt.value
                        )
                    )
                    {
                        imgClone.attributes.alt.value = local.alt.value;
                    }

                    local.text = imgClone.attributes.alt.value;
                    local.html = local.this._nodeToHtml(imgClone);
                }
            }

            local.unload = function ()
            {
                local.unbind();
            }

            local.unbind = function ()
            {
                local.object.disconnect();
            }

            local.focus = function ()
            {
                local.container.dispatchEvent( local.this._focus_event );
                window.scroll(window.scrollX,local.container.offsetTop);
            }
        };

        FieldDrupal7Image.prototype = SiteAttentionModule.FieldAbstract.prototype;

        FieldDrupal7Image.prototype.constructor = FieldDrupal7Image;

        return FieldDrupal7Image;
    };

    var local =
    {
        document : undefined,
        resize : undefined,
        click: undefined,
        container : undefined,
        header : undefined,
        overlay : undefined,
        content : undefined,
        adminMenu : undefined,
        published : undefined,
        url: undefined,
        urlTab: undefined,
        summary: undefined,
        summaryTab: undefined,
        summaryToggle: undefined
    }

    var getPid = function ()
    {
        return Drupal.settings.SiteAttention.nodeId;
    }

    var getIid = function ()
    {
        return Drupal.settings.SiteAttention.iid;
    }

    var getType = function ()
    {
        return Drupal.settings.SiteAttention.nodeType;
    }

    var getLang = function ()
    {
        return Drupal.settings.SiteAttention.nodeLang;
    }

    var getUserName = function ()
    {
        return Drupal.settings.SiteAttention.userName;
    }

    var getUserId = function ()
    {
        return Drupal.settings.SiteAttention.userId;
    }

    var getUrl = function ()
    {
        return Drupal.settings.SiteAttention.nodeUrl;
    }

    var getPublished = function ()
    {
        return Drupal.settings.SiteAttention.nodeStatus;
    }

    var getFields = function ()
    {
        var imageWidget = '.field-widget-image-image',
            path = '#edit-path-alias',
            title = '#edit-title',
            teaser = '#edit-field-teaser-und-0-value',
            summary = '#edit-body-und-0-summary',
            body = '#edit-body-und-0-value',
            fields = [
            {
                seo: 'url',
                name: 'URL alias',
                selector: path,
            },
            {
                seo: 'title',
                name: 'Title',
                selector:  title,
            },
            {
                seo: 'content',
                name: 'Title',
                selector: title,
                tag: 'h1'
            },
            {
                seo: 'content',
                name: 'Body',
                selector: body,
            },
        ];

        if (document.querySelector(teaser))
        {
            fields.push(
            {
                seo: 'content',
                name: 'Teaser',
                selector: teaser,
            },
            {
                seo: 'metadescription',
                name: 'Teaser',
                selector: teaser,
            });
        }

        if (document.querySelector(summary))
        {
            fields.push(
            {
                seo: 'metadescription',
                name: 'Summary',
                selector: summary,
            });
        }

        document.querySelectorAll(imageWidget).forEach(function(el)
        {
            fields.push(
            {
                seo: 'content',
                name: el.getElementsByTagName('label')[0].textContent,
                selector: '#' + el.id,
                type: FieldDrupal7Image
            });
        });

        return fields;
    }

    var getContainer = function ()
    {
        local.container = local.document.createElement('div');

        local.container.id = 'SAPL';

        // IE fix for now
        local.container.style.position = 'fixed';
        local.container.style.right = '0';
        local.container.style.bottom = '0';
        local.container.style.zIndex = '599';

        local.document.body.appendChild(local.container);

        return local.container;
    }

    var getMinimised = function ()
    {
        return false;
    }

    var onUpdateUi = function ()
    {
        return true;
    }

    var onMinimise = function ()
    {
        if ( local.overlay )
        {
            local.content.style.width = '';
        }

        else
        {
            if ( local.header )
            {
                local.header.classList.remove("SA_max_header");
            }

            if ( local.content )
            {
                local.content.classList.remove("SA_max_content");
            }
        }
    }

    var onMaximise = function ()
    {
        if ( local.overlay )
        {
            local.content.style.width
                = parent.innerWidth
                - SiteAttention.ui.main.frame.clientWidth
                + "px";
        }

        else
        {
            if ( local.header )
            {
                local.header.classList.add("SA_max_header");
            }

            if ( local.content )
            {
                local.content.classList.add("SA_max_content");
            }
        }
    }

    var onResize = function(local)
    {
        // is overlay and plugin is shown
        if ( local.overlay && local.content.style.width !== '' )
        {
            local.content.style.width
                = parent.innerWidth
                - SiteAttention.ui.main.frame.clientWidth
                + "px";
        }

        var bodyStyle = getComputedStyle(local.document.body);

        var height =
            local.document.defaultView.innerHeight -
            bodyStyle.paddingTop.replace('px','') - bodyStyle.marginTop.replace('px','');

        local.container.style.height = height + "px";
    }

    var onPlay = function ()
    {
        onResize(local);
        onMaximise();
    }

    var instanceOf = function(obj,type)
    {
        // Function.name poly
        !function(){function t(){var t,e;return this===Function||this===Function.prototype.constructor?e="Function":this!==Function.prototype&&(t=(""+this).match(n),e=t&&t[1]),e||""}var n=/^\s*function\s+([^\(\s]*)\s*/,e=!("name"in Function.prototype&&"name"in function(){}),o="function"==typeof Object.defineProperty&&function(){var t;try{Object.defineProperty(Function.prototype,"_xyz",{get:function(){return"blah"},configurable:!0}),t="blah"===Function.prototype._xyz,delete Function.prototype._xyz}catch(n){t=!1}return t}(),r="function"==typeof Object.prototype.__defineGetter__&&function(){var t;try{Function.prototype.__defineGetter__("_abc",function(){return"foo"}),t="foo"===Function.prototype._abc,delete Function.prototype._abc}catch(n){t=!1}return t}();Function.prototype._name=t,e&&(o?Object.defineProperty(Function.prototype,"name",{get:function(){var n=t.call(this);return this!==Function.prototype&&Object.defineProperty(this,"name",{value:n,configurable:!0}),n},configurable:!0}):r&&Function.prototype.__defineGetter__("name",function(){var n=t.call(this);return this!==Function.prototype&&this.__defineGetter__("name",function(){return n}),n}))}();

        if (!(type instanceof Function)) return false;

        return Object.prototype.toString.call(obj).slice(8,-1) === type.name;
    }

    var setHtmlEnv = function ()
    {
        switch ( Drupal.settings.ajaxPageState.theme )
        {
            case 'seven':
                parent.document.body.classList.add('siteattention-seven');
                local.header = "#branding";
                local.content = "#content";
                break;

            case 'garland':
                parent.document.body.classList.add('siteattention-garland');
                local.header = "#header";
                local.content = "#container";
                break;

            case 'bartik':
                parent.document.body.classList.add('siteattention-bartik');
                local.header = "#header";
                local.content = "#main";
                break;

            case 'stark':
                parent.document.body.classList.add('siteattention-stark');
                local.header = "#header";
                local.content = "#main-wrapper";
                break;

            case 'bootstrap':
                parent.document.body.classList.add('siteattention-bootstrap');
                local.header = "#navbar";
                local.content = ".main-container";
                break;

            case 'zen':
                parent.document.body.classList.add('siteattention-zen');
                local.header = ".header";
                local.content = ".layout-center";
                break;
        }

        // go on the parent window
        local.overlay = parent.document.querySelector ( '#overlay-container' );

        // set the document to the root window's document
        if ( local.overlay )
        {
            local.document = parent.document;
            local.content = local.overlay;

            // inject the css on the parent window where the container lays
            saInjectStyle(Drupal.settings.SiteAttention.cssUrl, undefined, local.document);

            // when page changes with ajax
            parent.addEventListener('popstate',function(){
                if(SiteAttention !== undefined)
                {
                    SiteAttention.quit();
                    onMinimise();
                }
            });

            // IE fix which doesn;t work from other iframes anyways
             parent.addEventListener('hashchange',function(){
                if(SiteAttention !== undefined)
                {
                    SiteAttention.quit();
                    onMinimise();
                }
            });
        }

        else
        {
            local.document = document;
            local.content = document.querySelector( local.content );
        }

        local.selectBodyFormat = document.getElementById('edit-body-und-0-format--2');
        local.published = document.querySelector ( '#edit-status' );
        local.header = document.querySelector( local.header );
        local.url = document.querySelector ( '#edit-path-alias' );
        local.summary = document.querySelector ( '#edit-body-und-0-summary' );
        local.summaryToggle = document.querySelector ( '.link-edit-summary' );
        local.summaryTab = document.querySelector ( '.text-summary-wrapper' );
        local.toolbar = local.document.querySelector ( '#toolbar' );
        local.adminMenu = local.document.querySelector ( '#admin-menu' );

        if (local.toolbar)
        {
            local.shortcut = local.toolbar.getElementsByClassName('toolbar-toggle-processed');
        }

        local.resize = document.createEvent('HTMLEvents');
        local.resize.initEvent('resize', true, true);
        local.resize.eventName = 'SiteAttention resize';

        local.click = document.createEvent('HTMLEvents');
        local.click.initEvent('click', true, true);
        local.click.eventName = 'SiteAttention click';

        var tabs = Array.prototype.slice.call ( document.querySelector ( '.vertical-tabs-panes' ).children ),
            pathTab = document.querySelector ( '#edit-path' ),
            tabIndex = tabs.indexOf( pathTab );

        local.urlTab = document.querySelector ( '.vertical-tabs-list' ) .children [ tabIndex ] .children [ 0 ];

        // when focus on the URL input, trigger click so container expands
        if ( local.url && local.urlTab )
        {
            local.url.addEventListener ( 'focus' , function ( event )
            {
                local.urlTab.dispatchEvent ( local.click );
            } );
        }

        // when focuds on summary, trigger click so it's container expands
        if ( local.summary && local.summaryToggle )
        {
            local.summary.addEventListener ( 'focus' , function ( event )
            {
                if ( local.summaryTab.style.display === 'none' )
                {
                    local.summaryToggle.dispatchEvent ( local.click );
                }
            } );
        }

        if (local.shortcut)
        {
            local.shortcut[0].addEventListener('click',function(){
                onResize(local);
            });
        }

        // body format changed
        if (local.selectBodyFormat){
            local.selectBodyFormat.addEventListener('change',function(event){
                SiteAttention.load({
                    cms: SiteAttentionModule.Cms.Drupal,
                    pid: getPid(),
                    iid: getIid(),
                    type: getType(),
                    lang: getLang(),
                    user: getUserName(),
                    url: getUrl(),
                    published: getPublished(),
                    map: SiteAttentionModule.FieldFactory(getFields())
                } );
            });
        }

        // bind resize event listener
        parent.addEventListener('resize',function(event)
        {
            onResize(local);
        });
    }

    var setLicense = function ( status, key, instance )
    {
        if ( key instanceof Object )
        {
            instance = key;
            key = undefined;
        }

        jQuery.ajax(
        {
            url: Drupal.settings.SiteAttention.urlSaveLicense,
            type: 'POST',
            dataType: 'json',   // response
            contentType: 'application/json', // request
            data: JSON.stringify
            ({
                key: key,
                iid: instance.iid,
                iname: instance.name,
                locked: instance.locked
            }),
            success: function ( data ) {}
        } );
    }

    var saSetHooks = function ()
    {
        SiteAttentionModule.hooks.add('after','register','Saving client data',setLicense);

        SiteAttentionModule.hooks.add('after','instance','Save instance data',setLicense);

        SiteAttentionModule.hooks.add('after','license','Save existing license',setLicense);

        SiteAttentionModule.hooks.add('after','update','UI update',onUpdateUi);

        SiteAttentionModule.hooks.add('after','minimise','Minimise',onMinimise);

        SiteAttentionModule.hooks.add('after','maximise','Maximise',onMaximise);

        SiteAttentionModule.hooks.add('after','play','Starting SiteAttention', onPlay);
    }

    var saInjectScript = function(url,cb,doc)
    {
        if (!instanceOf(url,String)) return;
        if (!instanceOf(doc,HTMLDocument)) doc = document;

        var script = doc.createElement('script');
        script.type = 'text/javascript';
        script.async = true;
        script.src = url;

        if (instanceOf(cb,Function))
            script.addEventListener('load',cb);

        doc.body.appendChild(script);
    }

    var saInjectStyle = function(url,cb,doc)
    {
        if (!instanceOf(url,String)) return;
        if (!instanceOf(doc,HTMLDocument)) doc = document;

        var style = doc.createElement('style');
        style.type = 'text/css';
        style.media = 'all';
        style.innerText = '@import url("'+url+'");';

        if (instanceOf(cb,Function))
            style.addEventListener('load',cb);

        doc.head.appendChild(style);
    }

    var saLoaded = function ( )
    {
        // Initialize the class
        FieldDrupal7Image = FieldDrupal7Image();

        // Figure out UI environment before SA.play()
        setHtmlEnv();

        // Set the hooks before SA.play()
        saSetHooks();

        SiteAttention.play(
        {
            container: getContainer(),
            minimised: getMinimised()
        } );

        SiteAttention.load(
        {
            cms: SiteAttentionModule.Cms.Drupal,
            pid: getPid(),
            iid: getIid(),
            type: getType(),
            lang: getLang(),
            user: getUserName(),
            url: getUrl(),
            published: getPublished(),
            map: SiteAttentionModule.FieldFactory(getFields())
        } );
    }

    saInjectScript( Drupal.settings.SiteAttention.apiUrl, saLoaded , local.document );
};

Drupal.behaviors.siteattention = {
    attach: function ( context, settings )
    {
        if (typeof SiteAttention === 'undefined'){
            SiteAttentionDrupal();
        }
    }
}